package nl.amis.crm.bpel.client;

import javax.naming.InitialContext;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.collaxa.xml.XMLHelper;

import com.oracle.bpel.client.ClientDefs;
import com.oracle.bpel.client.Locator;
import com.oracle.bpel.client.NormalizedMessage;
import com.oracle.bpel.client.ServerException;
import com.oracle.bpel.client.delivery.IDeliveryService;

import java.io.IOException;

import java.rmi.RemoteException;

import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;

import javax.naming.Context;

import nl.amis.crm.CustomerProfile;

public class CustomerProfileManager {

    public CustomerProfileManager() {
    }

    public static CustomerProfile findCustomerProfile(String customerId) throws ServerException, 
                                                                                RemoteException, 
                                                                                IOException {
        String xml = 
            "<CustomerProfileServiceProcessRequest xmlns=\"http://xmlns.oracle.com/CustomerProfileService\"><customerId>" + 
            customerId + 
            "</customerId></CustomerProfileServiceProcessRequest> ";
        // Set JNDI Properties for remote invocation ( will work for local also... 
        Hashtable jndi = null;
        jndi = new Hashtable();
        // Change the PROVIDER_URL to your BPEL PM host...
        jndi.put(Context.PROVIDER_URL, "ormi://localhost/orabpel");
        jndi.put(Context.INITIAL_CONTEXT_FACTORY, 
                 "com.evermind.server.rmi.RMIInitialContextFactory");
        // connect to the admin principal                 
        jndi.put(Context.SECURITY_PRINCIPAL, "admin");
        jndi.put(Context.SECURITY_CREDENTIALS, "welcome");
        jndi.put("dedicated.connection", "true");
        // call to default domain using (default) password bpel
        // note: if we run this code in the same JVM as the Oracle BPEL PM, then jndi can be null
        Locator locator = new Locator("default", "bpel", jndi);

        IDeliveryService deliveryService = 
            (IDeliveryService)locator.lookupService(IDeliveryService.SERVICE_NAME);

        // construct the normalized message and send to collaxa server
        NormalizedMessage nm = new NormalizedMessage();
        nm.addPart("payload", xml);

        NormalizedMessage res = 
            deliveryService.request("CustomerProfileService", "process", nm);

        Map payload = res.getPayload();

        //System.out.println("BPELProcess  CustomerProfileService executed!<br>");
        Element partEl = (Element)payload.get("payload");
        //System.out.println(partEl.getFirstChild().getNodeName());
        //System.out.println((partEl.getElementsByTagName("name")).item(0).getNodeValue());
        //System.out.println("Credit Rating is " + 
        //                   XMLHelper.elementToString(partEl));
        return buildCustomerProfile(partEl);
    }


    public static void main(String[] args) throws Exception {
        CustomerProfile profile = 
            CustomerProfileManager.findCustomerProfile("2");
        System.out.println("Customer Name: " + profile.getName());
    }

    private static CustomerProfile buildCustomerProfile(Element serviceResponse) {
        CustomerProfile profile = new CustomerProfile();
        Element contactDetails = (Element)(serviceResponse.getElementsByTagName("contactDetails")).item(0);
        profile.setName((contactDetails.getElementsByTagName("name")).item(0).getNodeValue());
        profile.setStreet((contactDetails.getElementsByTagName("street")).item(0).getNodeValue());
//        profile.setHouseNumber((contactDetails.getElementsByTagName("houseNumber")).item(0).getNodeValue());
        Element financialStatus = (Element)(serviceResponse.getElementsByTagName("financialStatus")).item(0);
        profile.setTotalRevenue(Float.parseFloat((financialStatus.getElementsByTagName("totalRevenue")).item(0).getNodeValue()));
        profile.setCreditLimit(Float.parseFloat((financialStatus.getElementsByTagName("creditLimit")).item(0).getNodeValue()));
              
        /*
 * <CustomerProfileServiceProcessResponse>
        <contactDetails>
        <name>Henk Willemsen</name>
        <street>Roodbruin</street>
        </contactDetails>
        <financialStatus/>
        </CustomerProfileServiceProcessResponse>
*/
        return profile;
    }
}
